@@ -1,8 +1,12 @@
 //! Commonly used listeners.
 
 mod combined;
+#[cfg(feature = "native-tls")]
+mod native_tls;
+#[cfg(feature = "rustls")]
+mod rustls;
 mod tcp;
-#[cfg(feature = "tls")]
+#[cfg(any(feature = "rustls", feature = "native-tls"))]
 mod tls;
 #[cfg(unix)]
 mod unix;
@@ -14,9 +18,13 @@ use std::{
 };
 
 pub use combined::{Combined, CombinedStream};
+#[cfg(feature = "native-tls")]
+pub use native_tls::{NativeTlsAcceptor, NativeTlsConfig, NativeTlsListener};
+#[cfg(feature = "rustls")]
+pub use rustls::{RustlsAcceptor, RustlsConfig, RustlsListener};
 pub use tcp::{TcpAcceptor, TcpListener};
-#[cfg(feature = "tls")]
-pub use tls::{IntoTlsConfigStream, TlsAcceptor, TlsConfig, TlsListener};
+#[cfg(any(feature = "rustls", feature = "native-tls"))]
+pub use tls::IntoTlsConfigStream;
 use tokio::io::{AsyncRead, AsyncWrite, ReadBuf, Result as IoResult};
 #[cfg(unix)]
 pub use unix::{UnixAcceptor, UnixListener};
@@ -63,15 +71,26 @@ pub trait AcceptorExt: Acceptor {
         Box::new(WrappedAcceptor(self))
     }
 
-    /// Consume this acceptor and return a new TLS acceptor.
-    #[cfg(feature = "tls")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
-    fn tls<S>(self, config_stream: S) -> TlsAcceptor<Self>
+    /// Consume this acceptor and return a new TLS acceptor with [`rustls`](https://crates.io/crates/rustls).
+    #[cfg(feature = "rustls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "rustls")))]
+    fn rustls<S>(self, config_stream: S) -> RustlsAcceptor<Self, S>
     where
         Self: Sized,
-        S: futures_util::Stream<Item = TlsConfig> + Send + 'static,
+        S: futures_util::Stream<Item = RustlsConfig> + Send + Unpin + 'static,
     {
-        TlsAcceptor::new(self, config_stream)
+        RustlsAcceptor::new(self, config_stream)
+    }
+
+    /// Consume this acceptor and return a new TLS acceptor with [`native-tls`](https://crates.io/crates/native-tls).
+    #[cfg(feature = "native-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "native-tls")))]
+    fn native_tls<S>(self, config_stream: S) -> NativeTlsAcceptor<Self, S>
+    where
+        Self: Sized,
+        S: futures_util::Stream<Item = NativeTlsConfig> + Send + Unpin + 'static,
+    {
+        NativeTlsAcceptor::new(self, config_stream)
     }
 }
 
@@ -105,15 +124,32 @@ pub trait Listener: Send {
         Combined::new(self, other)
     }
 
-    /// Consume this listener and return a new TLS listener.
-    #[cfg(feature = "tls")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
+    /// Consume this listener and return a new TLS listener with [`rustls`](https://crates.io/crates/rustls).
+    #[cfg(feature = "rustls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "rustls")))]
     #[must_use]
-    fn tls<S: IntoTlsConfigStream>(self, config_stream: S) -> TlsListener<Self, S>
+    fn rustls<S: IntoTlsConfigStream<RustlsConfig>>(
+        self,
+        config_stream: S,
+    ) -> RustlsListener<Self, S>
     where
         Self: Sized,
     {
-        TlsListener::new(self, config_stream)
+        RustlsListener::new(self, config_stream)
+    }
+
+    /// Consume this listener and return a new TLS listener with [`native-tls`](https://crates.io/crates/native-tls).
+    #[cfg(feature = "native-tls")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "native-tls")))]
+    #[must_use]
+    fn native_tls<S: IntoTlsConfigStream<NativeTlsConfig>>(
+        self,
+        config_stream: S,
+    ) -> NativeTlsListener<Self, S>
+    where
+        Self: Sized,
+    {
+        NativeTlsListener::new(self, config_stream)
     }
 }
 
@@ -201,26 +237,6 @@ mod tests {
     use super::{AcceptorExt, *};
     use crate::listener::TcpListener;
 
-    #[cfg(feature = "tls")]
-    #[tokio::test]
-    #[should_panic]
-    #[allow(unused_variables, unused_assignments)]
-    async fn test_box_acceptor() {
-        let mut a = TcpListener::bind("127.0.0.1:0")
-            .into_acceptor()
-            .await
-            .unwrap()
-            .boxed();
-
-        a = TcpListener::bind("127.0.0.1:0")
-            .tls(TlsConfig::new())
-            .combine(TcpListener::bind("127.0.0.1:0"))
-            .into_acceptor()
-            .await
-            .unwrap()
-            .boxed();
-    }
-
     #[tokio::test]
     async fn combined_listener() {
         let a = TcpListener::bind("127.0.0.1:0");
