@@ -1,6 +1,5 @@
 use std::{
     any::Any,
-    convert::TryInto,
     fmt::{self, Debug, Formatter},
 };
 
@@ -191,6 +190,19 @@ impl Response {
         &mut self.extensions
     }
 
+    /// Get a reference from extensions, similar to `self.extensions().get()`.
+    #[inline]
+    pub fn data<T: Send + Sync + 'static>(&self) -> Option<&T> {
+        self.extensions.get()
+    }
+
+    /// Inserts a value to extensions, similar to
+    /// `self.extensions().insert(data)`.
+    #[inline]
+    pub fn set_data(&mut self, data: impl Send + Sync + 'static) {
+        self.extensions.insert(data);
+    }
+
     /// Sets the body for this response.
     pub fn set_body(&mut self, body: impl Into<Body>) {
         self.body = body.into();
